"use strict";
module.exports = function (RED) {
    const debug = require('debug')('NRCHKB:HAPServiceNode');
    const HapNodeJS = require('hap-nodejs');
    const uuid = HapNodeJS.uuid;
    let publishTimers = {};
    const preInit = function (config) {
        RED.nodes.createNode(this, config);
        const node = this;
        node.RED = RED;
        const ServiceUtils = require('./utils/ServiceUtils')(node);
        new Promise((resolve) => {
            if (config.waitForSetupMsg) {
                debug('Waiting for Setup message. It should be of format {"payload":{"nrchkb":{"setup":{}}}}');
                node.setupDone = false;
                node.status({
                    fill: 'blue',
                    shape: 'dot',
                    text: 'Waiting for Setup',
                });
                node.handleWaitForSetup = (msg) => ServiceUtils.handleWaitForSetup(config, msg, resolve);
                node.on('input', node.handleWaitForSetup);
            }
            else {
                resolve(config);
            }
        }).then((newConfig) => {
            init.call(node, newConfig);
        });
    };
    const init = function (config) {
        this.config = config;
        const ServiceUtils = require('./utils/ServiceUtils')(this);
        this.isParentNode =
            typeof config.isParent === 'boolean' ? config.isParent : true;
        if (this.isParentNode) {
            debug('Starting Parent Service ' + config.name);
            configure(this, config);
        }
        else {
            ServiceUtils.waitForParent(this, config)
                .then(() => {
                debug('Starting ' +
                    (config.serviceName === 'CameraControl'
                        ? 'Camera'
                        : 'Linked') +
                    ' Service ' +
                    config.name);
                configure(this, config);
            })
                .catch(e => {
                this.status({
                    fill: 'red',
                    shape: 'ring',
                    text: 'Error while starting ' +
                        (config.serviceName === 'CameraControl'
                            ? 'Camera'
                            : 'Linked') +
                        ' Service',
                });
                this.error('Error while starting ' +
                    (config.serviceName === 'CameraControl'
                        ? 'Camera'
                        : 'Linked') +
                    ' Service ' +
                    config.name +
                    ': ', e);
            });
        }
    };
    const configure = function (node, config) {
        const Utils = require('./utils')(node);
        const AccessoryUtils = Utils.AccessoryUtils;
        const BridgeUtils = Utils.BridgeUtils;
        const CharacteristicUtils = Utils.CharacteristicUtils;
        const ServiceUtils = Utils.ServiceUtils;
        node.bridgeNode;
        let parentNode;
        node.parentService;
        if (node.isParentNode) {
            node.bridgeNode = RED.nodes.getNode(config.bridge);
            node.childNodes = [];
            node.childNodes.push(node);
        }
        else {
            parentNode = RED.nodes.getNode(config.parentService);
            if (!parentNode) {
                throw Error('Parent Node not assigned');
            }
            node.parentService = parentNode.service;
            if (!node.parentService) {
                throw Error('Parent Service not assigned');
            }
            node.bridgeNode = parentNode.bridgeNode;
            parentNode.childNodes.push(node);
        }
        node.name = config.name;
        node.topic = config.topic;
        node.filter = config.filter;
        node.serviceName = config.serviceName;
        node.manufacturer = config.manufacturer;
        node.serialNo = config.serialNo;
        node.model = config.model;
        node.accessoryType = config.accessoryType;
        node.firmwareRev = config.firmwareRev;
        node.hardwareRev = config.hardwareRev;
        node.softwareRev = config.softwareRev;
        const bridge = node.bridgeNode.bridge;
        const subtypeUUID = uuid.generate(node.id);
        const accessoryUUID = uuid.generate('A' +
            node.id +
            node.name +
            node.manufacturer +
            node.serialNo +
            node.model);
        let accessory;
        if (node.isParentNode) {
            accessory = AccessoryUtils.getOrCreate(bridge, {
                name: node.name,
                UUID: accessoryUUID,
                manufacturer: node.manufacturer,
                serialNo: node.serialNo,
                model: node.model,
                firmwareRev: node.firmwareRev,
                hardwareRev: node.hardwareRev,
                softwareRev: node.softwareRev,
            }, subtypeUUID);
            node.onIdentify = AccessoryUtils.onIdentify;
            accessory.on('identify', node.onIdentify);
        }
        else {
            accessory = parentNode.accessory;
        }
        const service = ServiceUtils.getOrCreate(accessory, {
            name: node.name,
            UUID: subtypeUUID,
            serviceName: node.serviceName,
            config: config,
        }, node.parentService);
        node.characteristicProperties = CharacteristicUtils.load(service, config);
        publishTimers = BridgeUtils.delayedPublish(node, publishTimers);
        node.service = service;
        node.status({
            fill: 'yellow',
            shape: 'ring',
            text: node.bridgeNode.pinCode,
        });
        node.supported = CharacteristicUtils.subscribeAndGetSupported(service);
        node.on('input', ServiceUtils.onInput);
        node.on('close', ServiceUtils.onClose);
        node.accessory = accessory;
    };
    return {
        preInit,
        init,
    };
};
