"use strict";
module.exports = function (node) {
    const debug = require('debug')('NRCHKB:ServiceUtils');
    const HapNodeJS = require('hap-nodejs');
    const Service = HapNodeJS.Service;
    const Characteristic = HapNodeJS.Characteristic;
    const CameraSource = require('../cameraSource').Camera;
    const NO_RESPONSE_MSG = 'NO_RESPONSE';
    const onCharacteristicGet = function (callback) {
        debug('onCharacteristicGet with status: ' + this.status + ' and value ' + this.value + ' and reachability is ' + node.accessory.reachable);
        callback(node.accessory.reachable === true ? this.status : new Error(NO_RESPONSE_MSG), this.value);
    };
    const onValueChange = function (outputNumber, { oldValue, newValue, context }) {
        const topic = node.topic ? node.topic : node.topic_in;
        const msg = { payload: {}, hap: {}, name: node.name, topic: topic };
        const key = this.displayName.replace(/ /g, '').replace(/\./g, '_');
        msg.payload[key] = newValue;
        if (context) {
            msg.hap = {
                oldValue: oldValue,
                newValue: newValue,
                context: context,
            };
        }
        node.status({
            fill: 'yellow',
            shape: 'dot',
            text: key + ': ' + newValue,
        });
        setTimeout(function () {
            node.status({});
        }, 3000);
        debug(node.name + ' received ' + key + ': ' + newValue);
        if (context || node.bridgeNode.allowMessagePassthrough) {
            if (outputNumber === 0) {
                node.send(msg);
            }
            else if (outputNumber === 1) {
                node.send([null, msg]);
            }
        }
    };
    const onCharacteristicSet = function (newValue, callback, context) {
        debug('onCharacteristicSet with status: ' + this.status + ' and value ' + this.value + ' and reachability is ' + node.accessory.reachable);
        callback(node.accessory.reachable === true ? null : new Error(NO_RESPONSE_MSG));
        onValueChange.call(this, 1, {
            undefined,
            newValue,
            context,
        });
    };
    const onCharacteristicChange = function ({ oldValue, newValue, context }) {
        onValueChange.call(this, 0, {
            oldValue,
            newValue,
            context,
        });
    };
    const onInput = function (msg) {
        if (msg.hasOwnProperty('payload')) {
            const type = typeof msg.payload;
            if (type !== 'object') {
                node.warn('Invalid payload type: ' + type);
                return;
            }
        }
        else {
            node.warn('Invalid message (payload missing)');
            return;
        }
        const topic = node.topic ? node.topic : node.name;
        if (node.filter === true && msg.topic !== topic) {
            debug('msg.topic doesn\'t match configured value and filter is enabled. Dropping message.');
            return;
        }
        let context = null;
        if (msg.payload.hasOwnProperty('Context')) {
            context = msg.payload.Context;
            delete msg.payload.Context;
        }
        node.topic_in = msg.topic ? msg.topic : '';
        Object.keys(msg.payload).map(function (key, index) {
            if (node.supported.indexOf(key) < 0) {
                node.warn('Try one of these characteristics: ' +
                    node.supported.join(', '));
            }
            else {
                node.accessory.updateReachability(msg.payload[key] !== NO_RESPONSE_MSG);
                let characteristic = node.service.getCharacteristic(Characteristic[key]);
                if (context !== null) {
                    characteristic.setValue(msg.payload[key], undefined, context);
                }
                else {
                    characteristic.setValue(msg.payload[key]);
                }
            }
        });
    };
    const onClose = function (removed, done) {
        const characteristics = node.service.characteristics.concat(node.service.optionalCharacteristics);
        characteristics.forEach(function (characteristic) {
            characteristic.removeListener('get', node.onCharacteristicGet);
            characteristic.removeListener('set', node.onCharacteristicSet);
            characteristic.removeListener('change', node.onCharacteristicChange);
        });
        if (node.isParentNode) {
            node.accessory.removeListener('identify', node.onIdentify);
        }
        if (removed) {
            if (node.isParentNode) {
                node.bridgeNode.bridge.removeBridgedAccessories([
                    node.accessory,
                ]);
                node.accessory.destroy();
            }
            else {
                node.accessory.removeService(node.service);
                node.parentService.removeLinkedService(node.service);
            }
        }
        else {
            node.accessory = null;
        }
        done();
    };
    const getOrCreate = function (accessory, serviceInformation, parentService) {
        let service = null;
        const newService = new Service[serviceInformation.serviceName](serviceInformation.name, serviceInformation.UUID);
        debug('Looking for service with UUID \'' + serviceInformation.UUID + '\'...');
        service = accessory.services.find(service => {
            return newService.subtype === service.subtype;
        });
        if (service && newService.UUID !== service.UUID) {
            debug('... service type changed! Removing the old service.');
            accessory.removeService(service);
            service = null;
        }
        if (!service) {
            debug('... didn\'t find it. Adding new ' +
                serviceInformation.serviceName +
                ' service.');
            if (serviceInformation.serviceName === 'CameraControl') {
                configureCameraSource(accessory, newService, serviceInformation.config);
                service = newService;
            }
            else {
                service = accessory.addService(newService);
            }
        }
        else {
            debug('... found it! Updating it.');
            service
                .getCharacteristic(Characteristic.Name)
                .setValue(serviceInformation.name);
        }
        if (parentService) {
            if (serviceInformation.serviceName === 'CameraControl') {
                debug('... and adding service to accessory.');
            }
            else {
                debug('... and linking service to parent.');
                parentService.addLinkedService(service);
            }
        }
        return service;
    };
    const configureCameraSource = function (accessory, service, config) {
        if (config.cameraConfigSource) {
            debug('Configuring Camera Source');
            if (!config.cameraConfigVideoProcessor) {
                node.error('Missing configuration for CameraControl: videoProcessor cannot be empty!');
            }
            else {
                accessory.configureCameraSource(new CameraSource(service, config, node));
            }
        }
        else {
            node.error('Missing configuration for CameraControl.');
        }
    };
    const waitForParent = (node, config) => {
        return new Promise((resolve) => {
            node.status({
                fill: 'blue',
                shape: 'dot',
                text: 'Waiting for Parent Service',
            });
            const checkAndWait = () => {
                if (node.RED.nodes.getNode(config.parentService)) {
                    resolve();
                }
                else {
                    setTimeout(checkAndWait, 1000);
                }
            };
            checkAndWait();
        });
    };
    const handleWaitForSetup = (config, msg, resolve) => {
        if (node.setupDone) {
            return;
        }
        if (msg.hasOwnProperty('payload')
            && (msg.payload.hasOwnProperty('nrchkb'))
            && msg.payload.nrchkb.hasOwnProperty('setup')) {
            node.setupDone = true;
            const newConfig = {
                ...config,
                ...msg.payload.nrchkb.setup
            };
            node.removeListener('input', node.handleWaitForSetup);
            resolve(newConfig);
        }
        else {
            node.warn('Invalid message (required {"payload":{"nrchkb":{"setup":{}}}})');
        }
    };
    return {
        getOrCreate,
        onCharacteristicGet,
        onCharacteristicSet,
        onCharacteristicChange,
        onInput,
        onClose,
        waitForParent,
        handleWaitForSetup
    };
};
